@@ -24,30 +24,19 @@ static blkid_tag blkid_new_tag(void)
 	if (!(tag = calloc(1, sizeof(struct blkid_struct_tag))))
 		return NULL;
 
+	DBG(TAG, ul_debugobj(tag, "alloc"));
 	INIT_LIST_HEAD(&tag->bit_tags);
 	INIT_LIST_HEAD(&tag->bit_names);
 
 	return tag;
 }
 
-void blkid_debug_dump_tag(blkid_tag tag)
-{
-	if (!tag) {
-		fprintf(stderr, "    tag: NULL\n");
-		return;
-	}
-
-	fprintf(stderr, "    tag: %s=\"%s\"\n", tag->bit_name, tag->bit_val);
-}
-
 void blkid_free_tag(blkid_tag tag)
 {
 	if (!tag)
 		return;
 
-	DBG(TAG, ul_debug("    freeing tag %s=%s", tag->bit_name,
-		   tag->bit_val ? tag->bit_val : "(NULL)"));
-	DBG(TAG, blkid_debug_dump_tag(tag));
+	DBG(TAG, ul_debugobj(tag, "freeing tag %s (%s)", tag->bit_name, tag->bit_val));
 
 	list_del(&tag->bit_tags);	/* list of tags for this device */
 	list_del(&tag->bit_names);	/* list of tags with this type */
@@ -66,9 +55,6 @@ blkid_tag blkid_find_tag_dev(blkid_dev dev, const char *type)
 {
 	struct list_head *p;
 
-	if (!dev || !type)
-		return NULL;
-
 	list_for_each(p, &dev->bid_tags) {
 		blkid_tag tmp = list_entry(p, struct blkid_struct_tag,
 					   bit_tags);
@@ -107,7 +93,7 @@ static blkid_tag blkid_find_head_cache(blkid_cache cache, const char *type)
 	list_for_each(p, &cache->bic_tags) {
 		tmp = list_entry(p, struct blkid_struct_tag, bit_tags);
 		if (!strcmp(tmp->bit_name, type)) {
-			DBG(TAG, ul_debug("    found cache tag head %s", type));
+			DBG(TAG, ul_debug("found cache tag head %s", type));
 			head = tmp;
 			break;
 		}
@@ -118,7 +104,7 @@ static blkid_tag blkid_find_head_cache(blkid_cache cache, const char *type)
 /*
  * Set a tag on an existing device.
  *
- * If value is NULL, then delete the tagsfrom the device.
+ * If value is NULL, then delete the tags from the device.
  */
 int blkid_set_tag(blkid_dev dev, const char *name,
 		  const char *value, const int vlength)
@@ -127,9 +113,6 @@ int blkid_set_tag(blkid_dev dev, const char *name,
 	char		*val = 0;
 	char		**dev_var = 0;
 
-	if (!dev || !name)
-		return -BLKID_ERR_PARAM;
-
 	if (value && !(val = strndup(value, vlength)))
 		return -BLKID_ERR_MEM;
 
@@ -155,6 +138,7 @@ int blkid_set_tag(blkid_dev dev, const char *name,
 			free(val);
 			return 0;
 		}
+		DBG(TAG, ul_debugobj(t, "update (%s) '%s' -> '%s'", t->bit_name, t->bit_val, val));
 		free(t->bit_val);
 		t->bit_val = val;
 	} else {
@@ -165,6 +149,7 @@ int blkid_set_tag(blkid_dev dev, const char *name,
 		t->bit_val = val;
 		t->bit_dev = dev;
 
+		DBG(TAG, ul_debugobj(t, "setting (%s) '%s'", t->bit_name, t->bit_val));
 		list_add_tail(&t->bit_tags, &dev->bid_tags);
 
 		if (dev->bid_cache) {
@@ -175,7 +160,7 @@ int blkid_set_tag(blkid_dev dev, const char *name,
 				if (!head)
 					goto errout;
 
-				DBG(TAG, ul_debug("    creating new cache tag head %s", name));
+				DBG(TAG, ul_debugobj(head, "creating new cache tag head %s", name));
 				head->bit_name = strdup(name);
 				if (!head->bit_name)
 					goto errout;
@@ -261,10 +246,10 @@ errout:
  *
  * These routines do not expose the list.h implementation, which are a
  * contamination of the namespace, and which force us to reveal far, far
- * too much of our internal implemenation.  I'm not convinced I want
+ * too much of our internal implementation.  I'm not convinced I want
  * to keep list.h in the long term, anyway.  It's fine for kernel
  * programming, but performance is not the #1 priority for this
- * library, and I really don't like the tradeoff of type-safety for
+ * library, and I really don't like the trade-off of type-safety for
  * performance for this application.  [tytso:20030125.2007EST]
  */
 
