@@ -26,7 +26,7 @@
  */
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <time.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -117,22 +117,44 @@ enum {
 
 enum {
 	LAST_TIMEFTM_NONE = 0,
-	LAST_TIMEFTM_SHORT_CTIME,
-	LAST_TIMEFTM_FULL_CTIME,
-	LAST_TIMEFTM_ISO8601
+	LAST_TIMEFTM_SHORT,
+	LAST_TIMEFTM_CTIME,
+	LAST_TIMEFTM_ISO8601,
+
+	LAST_TIMEFTM_HHMM,	/* non-public */
 };
 
 struct last_timefmt {
 	const char *name;
-	int in;
-	int out;
+	int in_len;	/* log-in */
+	int in_fmt;
+	int out_len;	/* log-out */
+	int out_fmt;
 };
 
 static struct last_timefmt timefmts[] = {
-	[LAST_TIMEFTM_NONE]	   = { "notime", 0, 0 },
-	[LAST_TIMEFTM_SHORT_CTIME] = { "short", 16, 7},
-	[LAST_TIMEFTM_FULL_CTIME]  = { "full",  24, 26},
-	[LAST_TIMEFTM_ISO8601]	   = { "iso", 24, 26}
+	[LAST_TIMEFTM_NONE] = { .name = "notime" },
+	[LAST_TIMEFTM_SHORT] = {
+		.name    = "short",
+		.in_len  = 16,
+		.out_len = 7,
+		.in_fmt  = LAST_TIMEFTM_CTIME,
+		.out_fmt = LAST_TIMEFTM_HHMM
+	},
+	[LAST_TIMEFTM_CTIME] = {
+		.name    = "full",
+		.in_len  = 24,
+		.out_len = 26,
+		.in_fmt  = LAST_TIMEFTM_CTIME,
+		.out_fmt = LAST_TIMEFTM_CTIME
+	},
+	[LAST_TIMEFTM_ISO8601] = {
+		.name    = "iso",
+		.in_len  = 24,
+		.out_len = 26,
+		.in_fmt  = LAST_TIMEFTM_ISO8601,
+		.out_fmt = LAST_TIMEFTM_ISO8601
+	}
 };
 
 /* Global variables */
@@ -141,15 +163,15 @@ static time_t lastdate;		/* Last date we've seen */
 static time_t currentdate;	/* date when we started processing the file */
 
 /* --time-format=option parser */
-static int which_time_format(const char *optarg)
+static int which_time_format(const char *s)
 {
 	size_t i;
 
 	for (i = 0; i < ARRAY_SIZE(timefmts); i++) {
-		if (strcmp(timefmts[i].name, optarg) == 0)
+		if (strcmp(timefmts[i].name, s) == 0)
 			return i;
 	}
-	errx(EXIT_FAILURE, _("unknown time format: %s"), optarg);
+	errx(EXIT_FAILURE, _("unknown time format: %s"), s);
 }
 
 /*
@@ -312,38 +334,26 @@ static int dns_lookup(char *result, int size, int useip, int32_t *a)
 	return getnameinfo(sa, salen, result, size, NULL, 0, flags);
 }
 
-static int time_formatter(const struct last_control *ctl, char *dst,
-			  size_t dlen, time_t *when, int pos)
+static int time_formatter(int fmt, char *dst, size_t dlen, time_t *when)
 {
-	struct tm *tm;
 	int ret = 0;
 
-	switch (ctl->time_fmt) {
+	switch (fmt) {
 	case LAST_TIMEFTM_NONE:
 		*dst = 0;
 		break;
-	case LAST_TIMEFTM_SHORT_CTIME:
-		if (pos == 0)
-			ret = sprintf(dst, "%s", ctime(when));
-		else {
-			tm = localtime(when);
-			if (!strftime(dst, dlen, "- %H:%M", tm))
-				ret = -1;
-		}
+	case LAST_TIMEFTM_HHMM:
+	{
+		struct tm *tm = localtime(when);
+		if (!snprintf(dst, dlen, "%02d:%02d", tm->tm_hour, tm->tm_min))
+			ret = -1;
 		break;
-	case LAST_TIMEFTM_FULL_CTIME:
-		if (pos == 0)
-			ret = sprintf(dst, "%s", ctime(when));
-		else
-			ret = sprintf(dst, "- %s", ctime(when));
+	}
+	case LAST_TIMEFTM_CTIME:
+		ret = snprintf(dst, dlen, "%s", ctime(when));
 		break;
 	case LAST_TIMEFTM_ISO8601:
-		tm = localtime(when);
-		if (pos == 0) {
-			if (!strftime(dst, dlen, "%Y-%m-%dT%H:%M:%S%z", tm))
-				ret = -1;
-		} else if (!strftime(dst, dlen, "- %Y-%m-%dT%H:%M:%S%z", tm))
-			ret = -1;
+		ret = strtime_iso(when, ISO_8601_DATE|ISO_8601_TIME|ISO_8601_TIMEZONE, dst, dlen);
 		break;
 	default:
 		abort();
@@ -412,6 +422,8 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 	/*
 	 *	Calculate times
 	 */
+	fmt = &timefmts[ctl->time_fmt];
+
 	utmp_time = p->UL_UT_TIME;
 
 	if (ctl->present) {
@@ -420,27 +432,36 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 		if (0 < logout_time && logout_time < ctl->present)
 			return 0;
 	}
-	if (time_formatter(ctl, &logintime[0], sizeof(logintime), &utmp_time, 0) < 0 ||
-	    time_formatter(ctl, &logouttime[0], sizeof(logouttime), &logout_time, 1) < 0)
+
+	/* log-in time */
+	if (time_formatter(fmt->in_fmt, logintime,
+			   sizeof(logintime), &utmp_time) < 0)
 		errx(EXIT_FAILURE, _("preallocation size exceeded"));
 
+	/* log-out time */
 	secs  = logout_time - utmp_time;
 	mins  = (secs / 60) % 60;
 	hours = (secs / 3600) % 24;
 	days  = secs / 86400;
 
+	strcpy(logouttime, "- ");
+	if (time_formatter(fmt->out_fmt, logouttime + 2,
+			   sizeof(logouttime) - 2, &logout_time) < 0)
+		errx(EXIT_FAILURE, _("preallocation size exceeded"));
+
 	if (logout_time == currentdate) {
-		if (ctl->time_fmt > LAST_TIMEFTM_SHORT_CTIME) {
+		if (ctl->time_fmt > LAST_TIMEFTM_SHORT) {
 			sprintf(logouttime, "  still running");
 			length[0] = 0;
 		} else {
 			sprintf(logouttime, "  still");
 			sprintf(length, "running");
 		}
-	} else if (days)
+	} else if (days) {
 		sprintf(length, "(%d+%02d:%02d)", days, hours, mins);
-	else
+	} else {
 		sprintf(length, " (%02d:%02d)", hours, mins);
+	}
 
 	switch(what) {
 		case R_CRASH:
@@ -450,7 +471,7 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 			sprintf(logouttime, "- down ");
 			break;
 		case R_NOW:
-			if (ctl->time_fmt > LAST_TIMEFTM_SHORT_CTIME) {
+			if (ctl->time_fmt > LAST_TIMEFTM_SHORT) {
 				sprintf(logouttime, "  still logged in");
 				length[0] = 0;
 			} else {
@@ -459,10 +480,10 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 			}
 			break;
 		case R_PHANTOM:
-			if (ctl->time_fmt > LAST_TIMEFTM_SHORT_CTIME) {
+			if (ctl->time_fmt > LAST_TIMEFTM_SHORT) {
 				sprintf(logouttime, "  gone - no logout");
 				length[0] = 0;
-			} else if (ctl->time_fmt == LAST_TIMEFTM_SHORT_CTIME) {
+			} else if (ctl->time_fmt == LAST_TIMEFTM_SHORT) {
 				sprintf(logouttime, "   gone");
 				sprintf(length, "- no logout");
 			} else {
@@ -470,13 +491,12 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 				sprintf(length, "no logout");
 			}
 			break;
-		case R_REBOOT:
-			break;
 		case R_TIMECHANGE:
 			logouttime[0] = 0;
 			length[0] = 0;
 			break;
 		case R_NORMAL:
+		case R_REBOOT:
 			break;
 		default:
 			abort();
@@ -495,7 +515,6 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 		strncat(domain, p->ut_host, len);
 	}
 
-	fmt = &timefmts[ctl->time_fmt];
 
 	if (ctl->showhost) {
 		if (!ctl->altlist) {
@@ -503,20 +522,20 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 				"%-8.*s %-12.12s %-16.*s %-*.*s %-*.*s %s\n",
 				ctl->name_len, p->ut_user, utline,
 				ctl->domain_len, domain,
-				fmt->in, fmt->in, logintime, fmt->out, fmt->out,
+				fmt->in_len, fmt->in_len, logintime, fmt->out_len, fmt->out_len,
 				logouttime, length);
 		} else {
 			len = snprintf(final, sizeof(final),
 				"%-8.*s %-12.12s %-*.*s %-*.*s %-12.12s %s\n",
 				ctl->name_len, p->ut_user, utline,
-				fmt->in, fmt->in, logintime, fmt->out, fmt->out,
+				fmt->in_len, fmt->in_len, logintime, fmt->out_len, fmt->out_len,
 				logouttime, length, domain);
 		}
 	} else
 		len = snprintf(final, sizeof(final),
 			"%-8.*s %-12.12s %-*.*s %-*.*s %s\n",
 			ctl->name_len, p->ut_user, utline,
-			fmt->in, fmt->in, logintime, fmt->out, fmt->out,
+			fmt->in_len, fmt->in_len, logintime, fmt->out_len, fmt->out_len,
 			logouttime, length);
 
 #if defined(__GLIBC__)
@@ -543,7 +562,7 @@ static int list(const struct last_control *ctl, struct utmp *p, time_t logout_ti
 }
 
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(const struct last_control *ctl, FILE *out)
 {
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(
@@ -557,7 +576,7 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_(" -a, --hostlast       display hostnames in the last column\n"), out);
 	fputs(_(" -d, --dns            translate the IP number back into a hostname\n"), out);
 	fprintf(out,
-	      _(" -f, --file <file>    use a specific file instead of %s\n"), _PATH_WTMP);
+	      _(" -f, --file <file>    use a specific file instead of %s\n"), ctl->lastb ? _PATH_BTMP : _PATH_WTMP);
 	fputs(_(" -F, --fulltimes      print full login and logout times and dates\n"), out);
 	fputs(_(" -i, --ip             display IP numbers in numbers-and-dots notation\n"), out);
 	fputs(_(" -n, --limit <number> how many lines to show\n"), out);
@@ -586,7 +605,7 @@ static int is_phantom(const struct last_control *ctl, struct utmp *ut)
 
 	if (ut->UL_UT_TIME < ctl->boot_time.tv_sec)
 		return 1;
-	pw = getpwnam(ut->ut_name);
+	pw = getpwnam(ut->ut_user);
 	if (!pw)
 		return 1;
 	sprintf(path, "/proc/%u/loginuid", ut->ut_pid);
@@ -616,7 +635,7 @@ static int is_phantom(const struct last_control *ctl, struct utmp *ut)
 static void process_wtmp_file(const struct last_control *ctl,
 			      const char *filename)
 {
-	FILE *fp;		/* Filepointer of wtmp file */
+	FILE *fp;		/* File pointer of wtmp file */
 
 	struct utmp ut;		/* Current utmp entry */
 	struct utmplist *ulist = NULL;	/* All entries */
@@ -839,7 +858,7 @@ static void process_wtmp_file(const struct last_control *ctl,
 			break;
 
 		default:
-			warnx("unrecogized ut_type: %d", ut.ut_type);
+			warnx("unrecognized ut_type: %d", ut.ut_type);
 		}
 
 		/*
@@ -872,7 +891,7 @@ int main(int argc, char **argv)
 	struct last_control ctl = {
 		.showhost = TRUE,
 		.name_len = LAST_LOGIN_LEN,
-		.time_fmt = LAST_TIMEFTM_SHORT_CTIME,
+		.time_fmt = LAST_TIMEFTM_SHORT,
 		.domain_len = LAST_DOMAIN_LEN
 	};
 	char **files = NULL;
@@ -911,7 +930,10 @@ int main(int argc, char **argv)
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	atexit(close_stdout);
-
+	/*
+	 * Which file do we want to read?
+	 */
+	ctl.lastb = strcmp(program_invocation_short_name, "lastb") == 0 ? 1 : 0;
 	while ((c = getopt_long(argc, argv,
 			"hVf:n:RxadFit:p:s:0123456789w", long_opts, NULL)) != -1) {
 
@@ -919,7 +941,7 @@ int main(int argc, char **argv)
 
 		switch(c) {
 		case 'h':
-			usage(stdout);
+			usage(&ctl, stdout);
 			break;
 		case 'V':
 			printf(UTIL_LINUX_VERSION);
@@ -948,7 +970,7 @@ int main(int argc, char **argv)
 			ctl.altlist = 1;
 			break;
 		case 'F':
-			ctl.time_fmt = LAST_TIMEFTM_FULL_CTIME;
+			ctl.time_fmt = LAST_TIMEFTM_CTIME;
 			break;
 		case 'p':
 			if (parse_timestamp(optarg, &p) < 0)
@@ -979,7 +1001,7 @@ int main(int argc, char **argv)
 			ctl.time_fmt = which_time_format(optarg);
 			break;
 		default:
-			usage(stderr);
+			usage(&ctl, stderr);
 			break;
 		}
 	}
@@ -987,10 +1009,6 @@ int main(int argc, char **argv)
 	if (optind < argc)
 		ctl.show = argv + optind;
 
-	/*
-	 * Which file do we want to read?
-	 */
-	ctl.lastb = strcmp(program_invocation_short_name, "lastb") == 0 ? 1 : 0;
 	if (!files) {
 		files = xmalloc(sizeof(char *));
 		files[nfiles++] = xstrdup(ctl.lastb ? _PATH_BTMP : _PATH_WTMP);
