@@ -371,9 +371,8 @@ static void server_loop(const char *socket_path, const char *pidfile_path,
 			if (ftruncate(fd_pidfile, 0))
 				err(EXIT_FAILURE, _("could not truncate file: %s"), pidfile_path);
 			write_all(fd_pidfile, reply_buf, strlen(reply_buf));
-			if (fd_pidfile > 1)
-				if (close_fd(fd_pidfile) != 0) /* Unlock the pid file */
-					err(EXIT_FAILURE, _("write failed: %s"), pidfile_path);
+			if (fd_pidfile > 1 && close_fd(fd_pidfile) != 0)
+				err(EXIT_FAILURE, _("write failed: %s"), pidfile_path);
 		}
 
 	}
@@ -421,7 +420,7 @@ static void server_loop(const char *socket_path, const char *pidfile_path,
 			warn(_("poll failed"));
 				all_done(uuidd_cxt, EXIT_FAILURE);
 		}
-		if (ret == 0) {		/* truen when poll() times out */
+		if (ret == 0) {		/* true when poll() times out */
 			if (uuidd_cxt->debug)
 				fprintf(stderr, _("timeout [%d sec]\n"), uuidd_cxt->timeout),
 			all_done(uuidd_cxt, EXIT_SUCCESS);
@@ -544,7 +543,6 @@ int main(int argc, char **argv)
 {
 	const char	*socket_path = UUIDD_SOCKET_PATH;
 	const char	*pidfile_path = NULL;
-	const char	*pidfile_path_param = NULL;
 	const char	*err_context = NULL;
 	char		buf[1024], *cp;
 	char		str[UUID_STR_LEN];
@@ -602,7 +600,7 @@ int main(int argc, char **argv)
 						_("failed to parse --uuids"));
 			break;
 		case 'p':
-			pidfile_path_param = optarg;
+			pidfile_path = optarg;
 			break;
 		case 'P':
 			no_pid = 1;
@@ -647,10 +645,8 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (!no_pid && !pidfile_path_param)
+	if (!no_pid && !pidfile_path)
 		pidfile_path = UUIDD_PIDFILE_PATH;
-	else if (pidfile_path_param)
-		pidfile_path = pidfile_path_param;
 
 	/* custom socket path and socket-activation make no sense */
 	if (s_flag && uuidd_cxt.no_sock && !uuidd_cxt.quiet)
