@@ -10,10 +10,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include "linux_reboot.h"
+#include <unistd.h>
+#include <sys/reboot.h>
 #include "nls.h"
 #include "c.h"
 #include "closestream.h"
+#include "pathnames.h"
+#include "path.h"
+
+#define LINUX_REBOOT_CMD_CAD_ON 0x89ABCDEF
+#define LINUX_REBOOT_CMD_CAD_OFF 0x00000000
 
 static void __attribute__ ((__noreturn__)) usage(FILE * out)
 {
@@ -21,7 +27,7 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 	fprintf(out, _(" %s hard|soft\n"), program_invocation_short_name);
 
 	fprintf(out, USAGE_SEPARATOR);
-	fprintf(out, "Set the function of the Ctrl-Alt-Del combination.\n");
+	fprintf(out, _("Set the function of the Ctrl-Alt-Del combination.\n"));
 
 	fprintf(out, USAGE_OPTIONS);
 	fprintf(out, USAGE_HELP);
@@ -30,9 +36,51 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
+static int get_cad(void)
+{
+	uint64_t val = path_read_u64(_PATH_PROC_CTRL_ALT_DEL);
+
+	switch (val) {
+	case 0:
+		fputs("soft\n", stdout);
+		break;
+	case 1:
+		fputs("hard\n", stdout);
+		break;
+	default:
+		printf("%s hard\n", _("implicit"));
+		warnx(_("unexpected value in %s: %ju"), _PATH_PROC_CTRL_ALT_DEL, val);
+		return EXIT_FAILURE;
+	}
+	return EXIT_SUCCESS;
+}
+
+static int set_cad(const char *arg)
+{
+	unsigned int cmd;
+
+	if (geteuid()) {
+		warnx(_("You must be root to set the Ctrl-Alt-Del behavior"));
+		return EXIT_FAILURE;
+	}
+	if (!strcmp("hard", arg))
+		cmd = LINUX_REBOOT_CMD_CAD_ON;
+	else if (!strcmp("soft", arg))
+		cmd = LINUX_REBOOT_CMD_CAD_OFF;
+	else {
+		warnx(_("unknown argument: %s"), arg);
+		return EXIT_FAILURE;
+	}
+	if (reboot(cmd) < 0) {
+		warnx("reboot");
+		return EXIT_FAILURE;
+	}
+	return EXIT_SUCCESS;
+}
+
 int main(int argc, char **argv)
 {
-	int ch;
+	int ch, ret;
 	static const struct option longopts[] = {
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
@@ -55,19 +103,9 @@ int main(int argc, char **argv)
 			usage(stderr);
 		}
 
-	if (geteuid())
-		errx(EXIT_FAILURE,
-		     _("You must be root to set the Ctrl-Alt-Del behavior"));
-
-	if (argc == 2 && !strcmp("hard", argv[1])) {
-		if (my_reboot(LINUX_REBOOT_CMD_CAD_ON) < 0)
-			err(EXIT_FAILURE, "reboot");
-	} else if (argc == 2 && !strcmp("soft", argv[1])) {
-		if (my_reboot(LINUX_REBOOT_CMD_CAD_OFF) < 0)
-			err(EXIT_FAILURE, "reboot");
-	} else {
-		usage(stderr);
-	}
-
-	return EXIT_SUCCESS;
+	if (argc < 2)
+		ret = get_cad();
+	else
+		ret = set_cad(argv[1]);
+	return ret;
 }
