@@ -80,6 +80,8 @@
 #include "pathnames.h"
 #include "strutils.h"
 #include "hwclock.h"
+#include "timeutils.h"
+#include "env.h"
 
 #ifdef HAVE_LIBAUDIT
 #include <libaudit.h>
@@ -137,7 +139,7 @@ int debug;
 bool badyear;
 
 /* User-specified epoch, used when rtc fails to return epoch. */
-unsigned long epoch_option = -1;
+unsigned long epoch_option = ULONG_MAX;
 
 /*
  * Almost all Award BIOS's made between 04/26/94 and 05/31/95 have a nasty
@@ -258,15 +260,12 @@ hw_clock_is_utc(const bool utc, const bool local_opt,
 static int read_adjtime(struct adjtime *adjtime_p)
 {
 	FILE *adjfile;
-	int rc;			/* local return code */
-	struct stat statbuf;	/* We don't even use the contents of this. */
 	char line1[81];		/* String: first line of adjtime file */
 	char line2[81];		/* String: second line of adjtime file */
 	char line3[81];		/* String: third line of adjtime file */
 	long timeval;
 
-	rc = stat(adj_file_name, &statbuf);
-	if (rc < 0 && errno == ENOENT) {
+	if (access(adj_file_name, R_OK) != 0) {
 		/* He doesn't have a adjtime file, so we'll use defaults. */
 		adjtime_p->drift_factor = 0;
 		adjtime_p->last_adj_time = 0;
@@ -400,15 +399,8 @@ mktime_tz(struct tm tm, const bool universal,
 	 */
 	zone = getenv("TZ");	/* remember original time zone */
 	if (universal) {
-		/* Set timezone to UTC as defined by the environment
-		 * variable TZUTC.  TZUTC undefined gives the default UTC
-		 * zonefile which usually does not take into account leap
-		 * seconds.  Define TZUTC to select your UTC zonefile which
-		 * does include leap seconds.  For example, with recent GNU
-		 * libc's:
-		 *    TZUTC=:/usr/share/zoneinfo/right/UTC
-		 */
-		setenv("TZ", getenv("TZUTC"), TRUE);
+		/* Set timezone to UTC */
+		xsetenv("TZ", "", TRUE);
 		/*
 		 * Note: tzset() gets called implicitly by the time code,
 		 * but only the first time. When changing the environment
@@ -443,7 +435,7 @@ mktime_tz(struct tm tm, const bool universal,
 	}
 	/* now put back the original zone. */
 	if (zone)
-		setenv("TZ", zone, TRUE);
+		xsetenv("TZ", zone, TRUE);
 	else
 		unsetenv("TZ");
 	tzset();
@@ -620,18 +612,18 @@ set_hardware_clock_exact(const time_t sethwtime,
 		if (ticksize < 0) {
 			if (debug)
 				printf(_("time jumped backward %.6f seconds "
-					 "to %ld.%06d - retargeting\n"),
+					 "to %ld.%06ld - retargeting\n"),
 				       ticksize, (long)nowsystime.tv_sec,
-				       (int)nowsystime.tv_usec);
+				       (long)nowsystime.tv_usec);
 			/* The retarget is handled at the end of the loop. */
 		} else if (deltavstarget < 0) {
 			/* deltavstarget < 0 if current time < target time */
 			if (debug >= 2)
-				printf(_("%ld.%06d < %ld.%06d (%.6f)\n"),
+				printf(_("%ld.%06ld < %ld.%06ld (%.6f)\n"),
 				       (long)nowsystime.tv_sec,
-				       (int)nowsystime.tv_usec,
+				       (long)nowsystime.tv_usec,
 				       (long)targetsystime.tv_sec,
-				       (int)targetsystime.tv_usec,
+				       (long)targetsystime.tv_usec,
 				       deltavstarget);
 			continue;  /* not there yet - keep spinning */
 		} else if (deltavstarget <= target_time_tolerance_secs) {
@@ -643,12 +635,12 @@ set_hardware_clock_exact(const time_t sethwtime,
 			 * aim for the next opportunity.
 			 */
 			if (debug)
-				printf(_("missed it - %ld.%06d is too far "
-					 "past %ld.%06d (%.6f > %.6f)\n"),
+				printf(_("missed it - %ld.%06ld is too far "
+					 "past %ld.%06ld (%.6f > %.6f)\n"),
 				       (long)nowsystime.tv_sec,
-				       (int)nowsystime.tv_usec,
+				       (long)nowsystime.tv_usec,
 				       (long)targetsystime.tv_sec,
-				       (int)targetsystime.tv_usec,
+				       (long)targetsystime.tv_usec,
 				       deltavstarget,
 				       target_time_tolerance_secs);
 			target_time_tolerance_secs += tolerance_incr_secs;
@@ -671,14 +663,14 @@ set_hardware_clock_exact(const time_t sethwtime,
 			    - RTC_SET_DELAY_SECS /* don't count this */
 			    + 0.5 /* for rounding */);
 	if (debug)
-		printf(_("%ld.%06d is close enough to %ld.%06d (%.6f < %.6f)\n"
-			 "Set RTC to %ld (%ld + %d; refsystime = %ld.%06d)\n"),
-		       (long)nowsystime.tv_sec, (int)nowsystime.tv_usec,
-		       (long)targetsystime.tv_sec, (int)targetsystime.tv_usec,
+		printf(_("%ld.%06ld is close enough to %ld.%06ld (%.6f < %.6f)\n"
+			 "Set RTC to %ld (%ld + %d; refsystime = %ld.%06ld)\n"),
+		       (long)nowsystime.tv_sec, (long)nowsystime.tv_usec,
+		       (long)targetsystime.tv_sec, (long)targetsystime.tv_usec,
 		       deltavstarget, target_time_tolerance_secs,
 		       (long)newhwtime, (long)sethwtime,
 		       (int)(newhwtime - sethwtime),
-		       (long)refsystime.tv_sec, (int)refsystime.tv_usec);
+		       (long)refsystime.tv_sec, (long)refsystime.tv_usec);
 
 	set_hardware_clock(newhwtime, universal, testing);
 }
@@ -697,13 +689,12 @@ display_time(const bool hclock_valid, struct timeval hwctime)
 		       "either invalid (e.g. 50th day of month) or beyond the range "
 		       "we can handle (e.g. Year 2095)."));
 	else {
-		struct tm *lt;
-		char *format = "%c";
-		char ctime_now[200];
+		char buf[ISO_8601_BUFSIZ];
 
-		lt = localtime(&hwctime.tv_sec);
-		strftime(ctime_now, sizeof(ctime_now), format, lt);
-		printf(_("%s  .%06d seconds\n"), ctime_now, (int)hwctime.tv_usec);
+		strtimeval_iso(&hwctime, ISO_8601_DATE|ISO_8601_TIME|ISO_8601_DOTUSEC|
+					 ISO_8601_TIMEZONE|ISO_8601_SPACE,
+					 buf, sizeof(buf));
+		printf("%s\n", buf);
 	}
 }
 
@@ -1062,8 +1053,8 @@ adjust_drift_factor(struct adjtime *adjtime_p,
 			drift_factor = 0;
 		} else {
 			if (debug)
-				printf(_("Clock drifted %.1f seconds in the past "
-					 "%.1f seconds\nin spite of a drift factor of "
+				printf(_("Clock drifted %f seconds in the past "
+					 "%f seconds\nin spite of a drift factor of "
 					 "%f seconds/day.\n"
 					 "Adjusting drift factor by %f seconds/day\n"),
 				       time_diff(nowtime, hclocktime),
@@ -1109,8 +1100,8 @@ calculate_adjustment(const double factor,
 			"Time since last adjustment is %d seconds\n",
 		       (int)(systime - last_time)),
 		       (int)(systime - last_time));
-		printf(_("Calculated Hardware Clock drift is %ld.%06d seconds\n"),
-		       (long)tdrift_p->tv_sec, (int)tdrift_p->tv_usec);
+		printf(_("Calculated Hardware Clock drift is %ld.%06ld seconds\n"),
+		       (long)tdrift_p->tv_sec, (long)tdrift_p->tv_usec);
 	}
 }
 
@@ -1267,7 +1258,7 @@ manipulate_clock(const bool show, const bool adjust, const bool noadjfile,
 		 const bool testing, const bool predict, const bool get)
 {
 	/* Contents of the adjtime file, or what they should be. */
-	struct adjtime adjtime;
+	struct adjtime adjtime = { 0 };
 	bool universal;
 	/* Set if user lacks necessary authorization to access the clock */
 	bool no_auth;
@@ -1459,13 +1450,13 @@ manipulate_epoch(const bool getepoch,
 			printf(_("Kernel is assuming an epoch value of %lu\n"),
 			       epoch);
 	} else if (setepoch) {
-		if (epoch_opt == -1)
+		if (epoch_opt == ULONG_MAX)
 			warnx(_
 			      ("To set the epoch value, you must use the 'epoch' "
 			       "option to tell to what value to set it."));
 		else if (testing)
 			printf(_
-			       ("Not setting the epoch to %d - testing only.\n"),
+			       ("Not setting the epoch to %lu - testing only.\n"),
 			       epoch_opt);
 		else if (set_epoch_rtc(epoch_opt))
 			printf(_
@@ -1532,6 +1523,7 @@ static int compare_clock (const bool utc, const bool local_opt)
 			printf("hw-time      system-time         freq-offset-ppm   tick\n");
 			printf("%10.0f   %10.6f\n", (double) time1_hw, time1_sys);
 		}
+		fflush(stdout);
 		sleep(10);
 	}
 
@@ -1639,8 +1631,6 @@ int main(int argc, char **argv)
 	 * fractions.
 	 */
 	time_t set_time = 0;	/* Time to which user said to set Hardware Clock */
-
-	bool permitted;		/* User is permitted to do the function */
 	int rc, c;
 
 	/* Variables set by various options; show may also be set later */
@@ -1910,28 +1900,6 @@ int main(int argc, char **argv)
 	      | setepoch | predict | compare | get))
 		show = 1;	/* default to show */
 
-	if (getuid() == 0)
-		permitted = TRUE;
-	else {
-		/* program is designed to run setuid (in some situations) */
-		if (set || systohc || adjust) {
-			warnx(_("Sorry, only the superuser can change "
-				"the Hardware Clock."));
-			permitted = FALSE;
-		} else if (systz || hctosys) {
-			warnx(_("Sorry, only the superuser can change "
-				"the System Clock."));
-			permitted = FALSE;
-		} else if (setepoch) {
-			warnx(_("Sorry, only the superuser can change the "
-				"Hardware Clock epoch in the kernel."));
-			permitted = FALSE;
-		} else
-			permitted = TRUE;
-	}
-
-	if (!permitted)
-		hwclock_exit(EX_NOPERM);
 
 #ifdef __linux__
 	if (getepoch || setepoch) {
@@ -1979,7 +1947,7 @@ void __attribute__((__noreturn__)) hwaudit_exit(int status)
 {
 	if (hwaudit_on) {
 		audit_log_user_message(hwaudit_fd, AUDIT_USYS_CONFIG,
-				       "changing system time", NULL, NULL, NULL,
+				       "op=change-system-time", NULL, NULL, NULL,
 				       status ? 0 : 1);
 		close(hwaudit_fd);
 	}
@@ -2034,11 +2002,6 @@ void __attribute__((__noreturn__)) hwaudit_exit(int status)
  * with any functions by these names, you will have unresolved external
  * references when you link.
  *
- * The program is designed to run setuid superuser, since we need to be able
- * to do direct I/O. (More to the point: we need permission to execute the
- * iopl() system call). (However, if you use one of the methods other than
- * direct ISA I/O to access the clock, no setuid is required).
- *
  * Here's some info on how we must deal with the time that elapses while
  * this program runs: There are two major delays as we run:
  *
