@@ -91,12 +91,12 @@ function ts_check_losetup {
 	ts_skip "no loop-device support"
 }
 
-function ts_skip_subtest {
+function ts_report_skip {
 	ts_report " SKIPPED ($1)"
 }
 
 function ts_skip {
-	ts_skip_subtest "$1"
+	ts_report_skip "$1"
 
 	ts_cleanup_on_exit
 	exit 0
@@ -130,7 +130,7 @@ function ts_failed {
 	exit $?
 }
 
-function ts_ok_subtest {
+function ts_report_ok {
 	if [ x"$1" == x"" ]; then
 		ts_report " OK"
 	else
@@ -139,7 +139,7 @@ function ts_ok_subtest {
 }
 
 function ts_ok {
-	ts_ok_subtest "$1"
+	ts_report_ok "$1"
 	exit 0
 }
 
@@ -367,11 +367,11 @@ function ts_init_py {
 
 function ts_valgrind {
 	if [ -z "$TS_VALGRIND_CMD" ]; then
-		$*
+		"$@"
 	else
 		$TS_VALGRIND_CMD --tool=memcheck --leak-check=full \
 				 --leak-resolution=high --num-callers=20 \
-				 --log-file="$TS_VGDUMP" $*
+				 --log-file="$TS_VGDUMP" "$@"
 	fi
 }
 
@@ -413,7 +413,7 @@ function ts_finalize_subtest {
 		ts_failed_subtest "$1"
 		res=1
 	else
-		ts_ok_subtest "$(tt_gen_mem_report "$1")"
+		ts_report_ok "$(tt_gen_mem_report "$1")"
 	fi
 
 	[ $res -ne 0 ] && TS_NSUBFAILED=$(( $TS_NSUBFAILED + 1 ))
@@ -424,6 +424,13 @@ function ts_finalize_subtest {
 	return $res
 }
 
+function ts_skip_subtest {
+	ts_report_skip "$1"
+	# reset environment back to parental test
+	ts_init_core_env
+
+}
+
 function ts_finalize {
 	ts_cleanup_on_exit
 
@@ -456,11 +463,13 @@ function ts_cleanup_on_exit {
 		ts_device_deinit "$dev"
 	done
 	unset TS_LOOP_DEVS
+
+	ts_scsi_debug_rmmod
 }
 
 function ts_image_md5sum {
 	local img=${1:-"$TS_OUTDIR/${TS_TESTNAME}.img"}
-	echo $(md5sum "$img" | awk '{printf $1}') $(basename "$img")
+	echo $("$TS_HELPER_MD5" < "$img") $(basename "$img")
 }
 
 function ts_image_init {
@@ -637,18 +646,15 @@ function ts_scsi_debug_init {
 		|| ts_skip "missing scsi_debug module (dry-run)"
 
 	# skip if still in use or removal of modules not supported at all
+	# We don't want a slow timeout here so we don't use ts_scsi_debug_rmmod!
 	modprobe -r scsi_debug &>/dev/null \
 		|| ts_skip "cannot remove scsi_debug module (rmmod)"
 
-	# TODO validate that all devices are gone, add function ts_scsi_debug_rmmod
-	# to be used by the tests too. Tests which produce non-removable scsi
-	# devices should fail!
-
 	modprobe -b scsi_debug "$@" &>/dev/null \
 		|| ts_skip "cannot load scsi_debug module (modprobe)"
 
 	# it might be still not loaded, modprobe.conf or whatever
-	lsmod | grep -q "^scsi_debug " \
+	lsmod 2>/dev/null | grep -q "^scsi_debug " \
 		|| ts_skip "scsi_debug module not loaded (lsmod)"
 
 	udevadm settle
@@ -664,7 +670,46 @@ function ts_scsi_debug_init {
 	TS_DEVICE="/dev/${devname}"
 
 	# TODO validate that device is really up, for now just a warning on stderr
-	test -b $TS_DEVICE || echo "warning: scsi_debug device is still down"
+	test -b $TS_DEVICE || echo "warning: scsi_debug device is still down" >&2
+}
+
+# automatically called once in ts_cleanup_on_exit()
+function ts_scsi_debug_rmmod {
+	local err=1
+	local t
+	local lastmsg
+
+	# Return early most importantly in case we are not root or the module does
+	# not exist at all.
+	[ $UID -eq 0 ] || return 0
+	[ -n "$TS_DEVICE" ] || return 0
+	lsmod 2>/dev/null | grep -q "^scsi_debug " || return 0
+
+	udevadm settle
+
+	# wait for successful rmmod if udevadm settle does not work
+	for t in 0 0.02 0.05 0.1 1; do
+		sleep $t
+		lastmsg="$(modprobe -r scsi_debug 2>&1)" && err=0 && break
+	done
+
+	if [ "$err" = "1" ]; then
+		ts_log "rmmod failed: '$lastmsg'"
+		ts_log "timeout removing scsi_debug module (rmmod)"
+		return 1
+	fi
+	if lsmod | grep -q "^scsi_debug "; then
+		ts_log "BUG! scsi_debug still loaded"
+		return 1
+	fi
+
+	# TODO Do we need to validate that all devices are gone?
+	udevadm settle
+	test -b "$TS_DEVICE" && echo "warning: scsi_debug device is still up" >&2
+
+	# TODO unset TS_DEVICE, check that nobody uses it later, e.g. ts_fdisk_clean
+
+	return 0
 }
 
 function ts_resolve_host {
@@ -719,3 +764,22 @@ function ts_init_socket_to_file {
 		ts_skip "socat socket stopped listening"
 	fi
 }
+
+function ts_has_mtab_support {
+	grep -q '#define USE_LIBMOUNT_SUPPORT_MTAB' ${top_builddir}/config.h
+	if [ $? == 0 ]; then
+		echo "yes"
+	else
+		echo "no"
+	fi
+}
+
+function ts_has_ncurses_support {
+	grep -q '#define HAVE_LIBNCURSES' ${top_builddir}/config.h
+	if [ $? == 0 ]; then
+		echo "yes"
+	else
+		echo "no"
+	fi
+}
+
